// Modul...: PAL.C
// Version.: 1.0
// Compiler: ICCAVR
// Datum...: Dezember 2005
// Autor...: Udo Jrss / Ulrich Radig
//------------------------------------------------------------------------------

#include "pal.h"

//-----------------------------------------------------------------------------

// P0.21 PWM5  Ausgang Sync  - [A]Diode[K] -     1k0 - 75 Ohm - Masse
// P0.19 MOSI  Ausgang Video - [A]Diode[K] - 330 Ohm /
//-----------------------------------------------------------------------------

volatile U08 pal_image[PAL_BYTES_PER_CHAR_ROW * PAL_CHAR_ROWS];

static volatile pU08 pal_image_ptr;
static volatile U32 pal_timer_count = 0; // Timer1 Zhlerwert
extern volatile U32 pal_timer_count0 = 0; // Timer1 Zhlerwert
static volatile U32 pal_line_count = 0; // Aktuelle Display-Zeile
//-----------------------------------------------------------------------------

//void pal_sync_timer_isr(void) __irq // Sync Interrupt-Service-Routine
void FIQ_Handler(void) __fiq 
{
  if ( ++pal_line_count < 311 ) // 1. Halbbild VSync	 311
  { 
    T1MR0 += PAL_HSYNC_64_US; // MatchRegister0 setzen
    PAL_SYNC_LOW(); // Sync ein
    pal_timer_count = T1TC + PAL_HSYNC_DURATION;
    while (T1TC < pal_timer_count) ; // Synchronpuls 4.8s 
    PAL_SYNC_HIGH(); // Sync aus

  	if (pal_line_count >= PAL_MIN_VERT_OFF && pal_line_count < PAL_MAX_VERT_OFF)
    {
	  pal_image_ptr = (pU08)&pal_image[(pal_line_count - PAL_MIN_VERT_OFF)*PAL_CHAR_COLS];
	  ssp_transfer(pal_image_ptr);
    }
  }
  else
  {	   
    T1MR0 += PAL_HSYNC_32_US; // MatchRegister0 setzen

    if (pal_line_count < 316) // 1. Halbbild VSync	  316
    {
      PAL_SYNC_LOW(); // Sync aus
      pal_timer_count = T1TC + PAL_HALF_HSYNC_DURATION;
      while (T1TC < pal_timer_count) ; // Synchronpuls 4.8s 
      PAL_SYNC_HIGH(); // Sync aus
    }
    else if (pal_line_count < 321) // 1. Halbbild VSync	 321
    {
      PAL_SYNC_HIGH(); // Sync aus
      pal_timer_count = T1TC + PAL_HALF_HSYNC_DURATION;
      while (T1TC < pal_timer_count) ; // Synchronpuls 4.8s 
      PAL_SYNC_LOW(); // Sync aus
    }
    else if (pal_line_count < 325) // 1. Halbbild VSync	 325
    {
      PAL_SYNC_LOW(); // Sync aus
      pal_timer_count = T1TC + PAL_HALF_HSYNC_DURATION;
      while (T1TC < pal_timer_count) ; // Synchronpuls 4.8s 
      PAL_SYNC_HIGH(); // Sync aus
    }
    else
    {
      T1MR0 += PAL_HSYNC_32_US; // MatchRegister0 setzen

      PAL_SYNC_LOW(); // Sync aus
      pal_timer_count = T1TC + PAL_HSYNC_DURATION;
      while (T1TC < pal_timer_count) ; // Synchronpuls 4.8s 
      PAL_SYNC_HIGH(); // Sync aus

      pal_line_count = 0;
    }
  }
  
  T1IR = 0xFF; // Timer1 Interrupt-Flags lschen
  VICVectAddr = 0; // Interrupt in VIC lschen
}
//-----------------------------------------------------------------------------

void pal_init(void)
{
  U32 n;

  for (n = 0; n < PAL_CHAR_ROWS * PAL_CHAR_LINES * PAL_CHAR_COLS; n++)
  {
    pal_image[n] = 0x00;
  }

  PAL_SYNC_INIT(); // Sync-Signal als Ausgang setzen

  T1TCR = 0; // Timer1 ausschalten
  T1PC = 0; // Prescaler = 0
  T1EMR = 0; // Match Pin Aktivitt ausschalten
  T1CCR = 0; // Capture ausschalten
  T1PR = (sys_get_fpclk() / PAL_TIMER_CLK) - 1; // Prescaler setzen
  T1MCR = TIM_MR0INT; // Freigabe fr Interrupt MR0 (Match-Register 0)
  T1MR0 = PAL_HSYNC_64_US; // MatchRegister0 setzen

  //sys_set_vectored_irq(pal_sync_timer_isr,PAL_SYNC_TIMER_SLOT,VIC_TIMER1);
  sys_set_fiq(VIC_TIMER1);
  T1TCR = TIM_CE; // Timer1 einschalten
}
//-----------------------------------------------------------------------------

void pal_encode_char(U32 col,U32 row,U08 chr)
{
  pU08 font_ptr;
  U32 n;

  if (chr >= 32)
  {
    chr -= 32;
  }

  font_ptr = (pU08)&FONT_IMAGE_7X9[chr][0];
  row *= PAL_BYTES_PER_CHAR_ROW;

  for (n = 0; n < PAL_CHAR_LINES; n++)
  {
    pal_image[row + col + n * PAL_CHAR_COLS] = *font_ptr++;
  }
}
//-----------------------------------------------------------------------------

void pal_encode_str(U32 col,U32 row,pU08 pstr)
{
  while (*pstr)
  {
    pal_encode_char(col++,row,*pstr++);
  }
}
//-----------------------------------------------------------------------------

void set_pixel (U32 zeile,U32 spalte)
{
 pal_image[(zeile*PAL_CHAR_COLS)+(spalte/8)]|=1 << (7- (spalte -((spalte/8)*8)));
}

//-----------------------------------------------------------------------------

void clear_pixel (U32 zeile,U32 spalte)
{
 pal_image[(zeile*PAL_CHAR_COLS) + (spalte/8)]&=~(1 << (7- (spalte -((spalte/8)*8))));
}

//-----------------------------------------------------------------------------
void logo (void)
{
  pU08 font_ptr;
  U32 x,y;

  font_ptr = (pU08) &LOGO[0];

 for (y = 150; y != 66; y --)
 {
  for (x = 6; x < 38; x++)
  {
    pal_image[x + (y * PAL_CHAR_COLS)] = ~(*font_ptr++);
  }
 }
}