#ifndef __SYSTEM_H
#define __SYSTEM_H
//------------------------------------------------------------------------------

#include "typedefs.h"
#include "uart0.h"
//------------------------------------------------------------------------------

#define SYS_ERROR_UNDEFINED_HANDLER "__EXCEPTION_UNDEFINED_HANDLER"
#define SYS_ERROR_UNDEFINED_INSTRUCTION "__EXCEPTION_UNDEFINED_INSTRUCTION"
#define SYS_ERROR_DATA_ABORT "__EXCEPTION_DATA_ABORT"
#define SYS_ERROR_PREFETCH_ABORT "__EXCEPTION_PREFETCH_ABORT"

#define SYS_RESET_ADDR (U32)0x00000000

// Oszillator- und Takt-Frequenzen
#define FOSC_MIN (U32)10000000
#define FOSC_MAX (U32)25000000

#define FCCLK_MIN (U32)10000000
#define FCCLK_MAX (U32)60000000

#define FCCO_MIN (U32)156000000
#define FCCO_MAX (U32)320000000

// PLL Definitionen
#define PLL_FEED_1 0xAA
#define PLL_FEED_2 0x55

#define PLL_PLLE BIT0
#define PLL_PLLC BIT1

#define PLL_STAT_PLLE BIT8
#define PLL_STAT_PLLC BIT9
#define PLL_STAT_PLOCK BIT10
//------------------------------------------------------------------------------

typedef enum
{
  VPB_DIV4 = 0, // fpclk = 1/4 fcclk
  VPB_DIV1,     // fpclk = fcclk
  VPB_DIV2      // fpclk = 1/2 fcclk
} tVPB_DIV;

typedef enum
{
  VIC_SLOT0 = 0, // hchste Prioritt
  VIC_SLOT1,
  VIC_SLOT2,
  VIC_SLOT3,
  VIC_SLOT4,
  VIC_SLOT5,
  VIC_SLOT6,
  VIC_SLOT7,
  VIC_SLOT8,
  VIC_SLOT9,
  VIC_SLOT10,
  VIC_SLOT11,
  VIC_SLOT12,
  VIC_SLOT13,
  VIC_SLOT14,
  VIC_SLOT15
} tVIC_IRQ_SLOT;
//------------------------------------------------------------------------------

void sys_init(void);
void sys_int_enable(U32 int_source);
void sys_int_disable(U32 int_source);
void sys_set_vectored_irq(void(*pISR)(),tVIC_IRQ_SLOT irq_slot,U32 int_souce);
void sys_set_fiq(U32 int_source);

U32 sys_get_fcclk(void); // PLL und Core-Clock zurckgeben
U32 sys_get_fpclk(void); // Peripherie-Clock zurckgeben

void sys_reset(void); // Reset ausfhren
//------------------------------------------------------------------------------

#endif
